unit QRLABELBAHTSTANG;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Quickrpt, Qrctrls;

type
  TControlBahtStang = Class(TPersistent)
  private
     FSymbolFront  : String;
     FSymbolBack   : String;
     FSymbolCredit : String;
     FCreditColor  : TColor;
  protected
     procedure SetSymbolCredit(Const Value : String);
     procedure SetSymbolFront(Const Value : String);
     procedure SetSymbolBack(Const Value : String);
     procedure SetCreditColor(Const Value :TColor);
   public
     Constructor Create;
     Destructor  Destroy;Override;
  published
     property SymbolFront   : String  read FSymbolFront   write SetSymbolFront;
     property SymbolBack    : String  read FSymbolBack    write SetSymbolBack;
     property SymbolCredit  : String  read FSymbolCredit  write SetSymbolCredit;
     property CreditColor   : TColor  read FCreditColor   write SetCreditColor;
end;

Type
    TQRLABELBAHTSTANG = class(TQRCustomLabel)
    private
       FControlBahtStang : TControlBahtStang;
       FEnaBled      : Boolean;
       FValue        : Double;
    protected
       procedure SetControlBahtStang(Const Value : TControlBahtStang);
       procedure SetEnabled(Const Value : Boolean);
       procedure SetValue(Value : Double);
       function  GetThai(Value:String;Position:Integer):String;
    public
       Constructor Create(Aowner : TComponent);Override;
       Destructor  Destroy;Override;
       function    GetReal(Value:Double):Double;
       function    GetDecimal(Value:Double):String;
       function    GetFormatStrtoFloat(Value:String):Double;
       function    NumericToBahtStang(Money : Double):String;
       procedure   Notification(AComponent : TComponent;Operation : TOperation);Override;
    published
       property ControlBahtStang : TControlBahtStang read FControlBahtStang write SetControlBahtStang;
       property Value         : Double  read FValue         write SetValue;
       property Enabled       : Boolean read FEnabled       write SetEnabled;
       property Alignment;
       property AlignToband;
       property AutoSize;
       property AutoStretch;
       property Caption;
       property Color;
       property Font;
       property ParentFont;
       property Transparent;
       property Wordwrap;

       property OnPrint;
  end;

procedure Register;

Var
    CountAll,
    CountReal,
    CountDecimal,
    I              : Integer;
    String2Process : String[20];
    Real           : String[20];
    Decimal        : String[2];
    ThaiBaht,
    ThaiStang,
    DecimalCh1,
    Strthai,
    DecimalCh2,
    BahtStang      : String;
    IsCredit       : Boolean;

implementation

//***************** TControlBahtStang *************//
procedure TControlBahtStang.SetCreditColor(Const Value : TColor);
begin
  FCreditColor := Value;
end;

procedure TControlBahtStang.SetSymbolCredit(Const Value : String);
begin
  FSymbolCredit := Value;
end;

procedure TControlBahtStang.SetSymbolFront(Const Value : String);
begin
  FSymbolFront := Value;
end;

procedure TControlBahtStang.SetSymbolBack(Const Value : String);
begin
   FSymbolBack := Value;
end;

Constructor TControlBahtStang.Create;
begin
   Inherited Create;
   CreditColor := ClRed;
   SymbolBack  := ')';
   SymbolCredit:= '-';
   SymbolFront := '(';
end;

Destructor TControlBahtStang.Destroy;
begin
   Inherited Destroy;
end;

//***************** TQRLABELBAHTSTANG *************//

procedure TQRLABELBAHTSTANG.SetControlBahtStang(Const Value : TControlBahtStang);
begin
   FControlBahtStang := Value
end;

procedure TQRLABELBAHTSTANG.SetEnabled(Const Value : Boolean);
begin
   FEnabled    := Value;
end;

procedure TQRLABELBAHTSTANG.SetValue(Value : Double);
begin
   FValue      := Value;
   If (FValue <> 0) and (FEnabled = True) Then
      If GetThai(FloattoStr(FValue),1) <> '(Դź) ' Then
         Font.Color:= clBlack;

  Caption   := FControlBahtStang.SymbolFront+NumericToBahtStang(Value)+FControlBahtStang.SymbolBack;
end;

function TQRLABELBAHTSTANG.GetFormatStrtoFloat(Value:String):Double;
// Change Format #,###,###.00 => #######.00
var A            : Integer;
    Str2Process2 : String[20];
    NewFormat    : String;
begin
   //Change Numeric to String
   For A := 1 to Length(Trim(Str2Process2)) Do
     begin
        If (Trim(Str2Process2[A]) <> ',') Then
            NewFormat := NewFormat+Str2Process2[A];
     end;
   Result     := StrtoFloat(NewFormat);
end;

function TQRLABELBAHTSTANG.GetReal(Value:Double):Double;
var  xReal : String[20];
begin
   IsCredit := False;
   Str(Value:15:2,String2Process );    //Change Numeric to String
   CountAll   := length(Trim(String2Process));  //Count String ->> CountAll
   If Value < 0 Then
      begin
        CountAll := CountAll-1;
        IsCredit := True;
        Font.Color := FControlBahtstang.CreditColor;
        xReal    := Copy(Trim(String2Process),2,CountAll-3);
      end
   else
      begin
        IsCredit := False;
        Font.Color := clBlack;
        xReal    := Copy(Trim(String2Process),1,CountAll-3);
      end;
   Result     := StrtoFloat(xReal);
end;

function TQRLABELBAHTSTANG.GetDecimal(Value:Double):String;
var xDecimal : String[2];
begin
   //Change Numeric to String
   Str(Value:15:2,String2Process );
   //Count String ->> CountAll
   CountAll   := length(Trim(String2Process));

   xDecimal   := Copy(Trim(String2Process),CountAll-1,CountAll);

   If Length(xDecimal) = 1 Then xDecimal := '0'+xDecimal;
   Result     := xDecimal
end;

function TQRLABELBAHTSTANG.NumericToBahtStang(Money : Double):String;
begin
{   //Change Numeric to String
   Str(Money:15:2,String2Process );
   //Count String ->> CountAll
   CountAll   := length(Trim(String2Process));
   Real       := Copy(Trim(String2Process),1,CountAll-3);
   Decimal    := Copy(Trim(String2Process),CountAll-1,CountAll);
 }
   I          := 0;
   Real       := FloattoStr(GetReal(Money));
   Decimal    := GetDecimal(Money);

   CountReal  := length(Real);

   ThaiBaht   := '';
   ThaiStang  := '';
   If IsCredit = True Then // Check Credit -###.00
      ThaiBaht:= FControlBahtStang.SymbolCredit;
   
//*******************************
//        Main Working
//*******************************
   For I := 1 to CountReal  Do // Loop = Real
     begin
       ThaiBaht := ThaiBaht+Getthai(Real[I],CountReal+1-I);
     end;
     ThaiBaht:=ThaiBaht+'ҷ';

// Check Decimal I
     DecimalCh1 := Copy(Decimal,1,1);
     if DecimalCh1 <>'0' Then
        begin
          If DecimalCh1 = '1' Then ThaiStang := '';
          If DecimalCh1 = '2' Then ThaiStang := '';
          If DecimalCh1 = '3' Then ThaiStang := '';
          If DecimalCh1 = '4' Then ThaiStang := '';
          If DecimalCh1 = '5' Then ThaiStang := '';
          If DecimalCh1 = '6' Then ThaiStang := 'ˡ';
          If DecimalCh1 = '7' Then ThaiStang := '';
          If DecimalCh1 = '8' Then ThaiStang := 'Ỵ';
          If DecimalCh1 = '9' Then ThaiStang := '';

          ThaiStang := thaiStang+'Ժ';
        end;
        
// Check Decimal II
      DecimalCh2 :=Copy(Decimal,2,1);
      If DecimalCh2 = '1' Then
         Begin
           if Copy(Decimal,1,1) ='0' Then
              ThaiStang := ThaiStang+'˹'
           else
              ThaiStang := ThaiStang+''
         end;
       If DecimalCh2 = '2' Then ThaiStang := ThaiStang+'ͧ';
       If DecimalCh2 = '3' Then ThaiStang := ThaiStang+'';
       If DecimalCh2 = '4' Then ThaiStang := ThaiStang+'';
       If DecimalCh2 = '5' Then ThaiStang := ThaiStang+'';
       If DecimalCh2 = '6' Then ThaiStang := ThaiStang+'ˡ';
       If DecimalCh2 = '7' Then ThaiStang := ThaiStang+'';
       If DecimalCh2 = '8' Then ThaiStang := ThaiStang+'Ỵ';
       If DecimalCh2 = '9' Then ThaiStang := ThaiStang+'';

     ThaiStang := ThaiStang+'ʵҧ';

// Check Lakh
     if (Copy(Decimal,1,1) <> '0') Or (Copy(Decimal,2,1) <> '0') Then
         Strthai := ThaiBaht+ThaiStang
     else
         Strthai := ThaiBaht+'ǹ';

    Result := Strthai
    // Return String Baht-Stang
end;

function TQRLABELBAHTSTANG.Getthai(Value:String;Position:Integer):String;
//--Check Real => Numeric => String
begin
  If Value = '0' Then
     begin
        if (CountAll <= 1) Or (FValue = 0) Then
            BahtStang :='ٹ'
        else
            BahtStang := '';
     end;
     
  If Value = '1' Then
     begin
        Case Position Of
          1,7,13 : begin
                     If (Position - CountReal <> 0 ) Or (Real[Position+1] = '0') Then
                         BahtStang :=''
                     else
                         BahtStang := '˹'
                   end;
          2,8,14 : BahtStang :='';
        else
                BahtStang :='˹';
        end
     end;

  If Value = '2' Then
     Begin
       if (Position = 2) Or (Position = 8)Or (Position = 14) Then
           BahtStang := ''
       else
           BahtStang :='ͧ'
     end;
  If Value = '3' Then BahtStang :='';
  If Value = '4' Then BahtStang :='';
  If Value = '5' Then BahtStang :='';
  If Value = '6' Then BahtStang :='ˡ';
  If Value = '7' Then BahtStang :='';
  If Value = '8' Then BahtStang :='Ỵ';
  If Value = '9' Then BahtStang :='';
// Check Unit of Position
If Value <> '0' Then
   begin
   Case Position Of
     7,13,19 : BahtStang := BahtStang+'ҹ';
     6,12,18 : BahtStang := BahtStang+'ʹ';
     5,11,17 : BahtStang := BahtStang+'';
     4,10,16 : BahtStang := BahtStang+'ѹ';
     3,9,15  : BahtStang := BahtStang+'';
     2,8,14  : BahtStang := BahtStang+'Ժ';
   end
   end
else
  If (Position = 7) Or (Position = 13)Or (Position =19) Then
      BahtStang := BahtStang+'ҹ';
 Result := BahtStang
end;

procedure TQRLABELBAHTSTANG.Notification(AComponent : TComponent;Operation : TOperation);
begin
   Inherited Notification(AComponent,Operation);
end;

Constructor TQRLABELBAHTSTANG.Create(Aowner : TComponent);
begin
   Inherited Create(AOwner);
   FControlBahtStang := TControlBahtStang.Create;
   Font.Name         := 'Ms Sans Serif';
   Enabled           := True;
end;

Destructor TQRLABELBAHTSTANG.Destroy;
begin
   Inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Warapetch Tools', [TQrLabelBahtStang]);
end;

end.
